/* 
 * AndSys 0.1.l.11 Copyright (C) 2012, 2013 Giulio Orru'
 * 
 * Licensed under the MIT license:
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package net.sourceforge.andsys;

import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.graphics.drawable.Drawable;

import java.io.File;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;


public class Package implements Comparable<Package> {

	private String mName; // Label or Name [USED IN equals METHOD]
	private String mPack; // Package Name [USED IN equals METHOD]
	private String mVers; // Version Name (not Number) [USED IN equals METHOD]
	private long mInst; // When the application was installed (Installed Date)

	private final PackageManager mPm; // For retrieving various kinds of
										// information related to the
										// application packages that are
										// currently installed on the device.
	private final PackageInfo mPi; // Overall information about the contents of
									// a package. This corresponds to all of the
									// information collected from
									// AndroidManifest.xml.

	/**
	 * Constructor
	 * 
	 * Warning: Constructors should not call an overridable method - that is,
	 * they should only call methods that are private, static, or final.
	 * 
	 * @param pm
	 *            PackageManager
	 * @param pi
	 *            PackageInfo
	 */
	public Package(PackageManager pm, PackageInfo pi) {
		this.mPm = pm;
		this.mPi = pi;
		this.mName = this.mPi.applicationInfo.loadLabel(this.mPm).toString();
		this.mPack = this.mPi.packageName;
		this.mVers = this.mPi.versionName;
		{
			// Full path to the location of this package
			String dirPath = this.mPi.applicationInfo.sourceDir;
			File dirResource = new File(dirPath);
			this.mInst = dirResource.lastModified();
		}
	}
	
	/**
	 * Destructor
	 */
	protected void finalize() throws Throwable{
		super.finalize();
		//this.mPm = The final fields cannot be assigned
		//this.mPi = The final fields cannot be assigned
		this.mName = null;
		this.mPack = null;
		this.mVers = null;
		this.mInst = 0;
	}

	public final String getName() {
		return this.mName;
	}

	public final String getPackage() {
		return this.mPack;
	}

	public final String getVersion() {
		return this.mVers;
	}

	public final long getInstalled() {
		return this.mInst;
	}

	public final Drawable getIcon() {
		Drawable icon;
		try {
			icon = this.mPm.getApplicationIcon(this.mPack);
		} catch (NameNotFoundException e) {
			icon = this.mPm.getDefaultActivityIcon();
		}
		return icon;
		/*
		 * { // ActivityInfo app = pm.getPackageInfo( this.getComponentName(),
		 * // PackageManager.GET_ACTIVITIES|PackageManager.GET_META_DATA); // if
		 * (pm.icon > 0) { // this.icon = activityInfo.icon; // } else {
		 * this.mIcon = pi.applicationInfo.icon; if (this.mIcon <= 0) {
		 * this.mIcon = context.getResources().getIdentifier(
		 * "default_activity_icon", "drawable", ""); } // this.mIcon =
		 * pi.applicationInfo.icon; // } }
		 */
	}

	/*
	 * Compares this instance with the specified object and indicates if they
	 * are equal.
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (obj == this) {
			return true;
		}
		if (obj == null || obj.getClass() != this.getClass()) {
			return false;
		}

		Package p = (Package) obj;
		return (mName == p.mName || (mName != null && mName.equals(p.mName)))
				&& (mPack == p.mPack || (mPack != null && mPack.equals(p.mPack)))
				&& (mVers == p.mVers || (mVers != null && mVers.equals(p.mVers)));
	}

	/*
	 * Returns an integer hash code for this object. By contract, any two
	 * objects for which equals(Object) returns true must return the same hash
	 * code value.
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int PRIME = 11;
		int result = 2;

		result = PRIME * result + ((mName == null) ? 0 : mName.hashCode());
		result = PRIME * result + ((mPack == null) ? 0 : mPack.hashCode());
		result = PRIME * result + ((mVers == null) ? 0 : mVers.hashCode());

		return result;
	}

	/*
	 * Returns a negative integer, zero, or a positive integer as this object is
	 * less than, equal to, or greater than the specified object.
	 * 
	 * Order by: mName, mPackage, mVersion
	 * 
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	@Override
	public int compareTo(Package another) {
		int compare = 0;

		compare = this.mName.compareTo(another.mName);
		if (compare == 0) {
			compare = this.mPack.compareTo(another.mPack);
			if (compare == 0) {
				compare = this.mVers.compareTo(another.mVers);
			}
		}

		return compare;
	}

	/**
	 * Make a list of installed Packages
	 * 
	 * @param context
	 * @return List of packages
	 */
	public static final List<Package> list(final Context context) {
		PackageManager pm = context.getPackageManager();
		List<PackageInfo> piList = pm.getInstalledPackages(0); // 0 = not system
		List<Package> packs = new ArrayList<Package>();
		for (int i = 0; i < piList.size(); i++) {
			PackageInfo pi = piList.get(i);
			Package ap;

			if (pi.versionName == null) {
				// system package
				continue;
			}
			ap = new Package(pm, pi);
			packs.add(ap);
		}
		return packs;
	}
	
	/**
	 * Order by "Name". A Comparator is used to compare two objects to determine
	 * their ordering with respect to each other.
	 */
	protected static class ComparatorName implements Comparator<Package> {
		@Override
		public int compare(Package lhs, Package rhs) {
			return lhs.getName().compareTo(rhs.getName());
		}
	}

	/**
	 * Order by "Package Name". A Comparator is used to compare two objects to
	 * determine their ordering with respect to each other.
	 */
	protected static class ComparatorPackage implements Comparator<Package> {
		@Override
		public int compare(Package lhs, Package rhs) {
			return lhs.getPackage().compareTo(rhs.getPackage());
		}
	}

	/**
	 * Order by "Installed Date". A Comparator is used to compare two objects to
	 * determine their ordering with respect to each other.
	 */
	protected static class ComparatorInstalled implements Comparator<Package> {
		@Override
		public int compare(Package lhs, Package rhs) {
			int r = 0;
			if (lhs.getInstalled() < rhs.getInstalled()) {
				r = -1;
			} else if (lhs.getInstalled() == rhs.getInstalled()) {
				r = 0;
			} else {
				r = 1;
			}
			return r;
		}
	}
}
